import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;


public class Conic {

    /**When this is negative, every vertical line intersects the
       conic defined by d. When it is positive, there are 2 vertical
       lines tangent to the hyperbolic.*/


    public static double[] disc(double[] d) {
	double d0=d[0];
	double d1=d[1];
	double d2=d[2];
	double d3=d[3];
	double d4=d[4];
	double d5=d[5];
	double D=d2*d2*d3*d3 - d1*d2*d3*d4 + d0*d2*d4*d4 + d1*d1*d2*d5 - 4*d0*d2*d2*d5;
	D=16*D;
	double[] DD={D,0,0};
	if(D>0) {
	  double r1=(4*d2*d3-2*d1*d4+Math.sqrt(D))/(2*d1*d1 - 8 *d0*d2);
	  double r2=(4*d2*d3-2*d1*d4-Math.sqrt(D))/(2*d1*d1 - 8 *d0*d2);
	  DD[1]=Math.min(r1,r2);
	  DD[2]=Math.max(r1,r2);

	}
	return DD;
    }

    /**This is a somewhat complicated rendering program.  When
       the distciminant is negative, every vertical hits the
       hyperbola. In this case we do one thing.  When the
       discriminant is positive, we need to compute the
       endpoints of the interval of verticals that do not
       hit the hyperbola.  After this, we need to pick out
       the special points on the hyperbola corresponding to
       the centers of the diagonals.*/

    public static Path2D.Double[] toPath(double[] d,Complex[] diag) {
	double[] D=disc(d);
	if(D[0]<0) return toPath1(d,diag);
	return toPath2(d,D,diag);
    }


    /**This is when all the verticals hit the conic*/

    public static Path2D.Double[] toPath1(double[] d,Complex [] diag) {
	Path2D.Double p=new Path2D.Double();
	for(int j=0;j<2;++j) {
	for(int i=0;i<4000;++i) {
	    double x=-20+1.0*i/100;
	    double[] y=rootsX(d,x);
	    if(y==null) return null;
	      if(i==0) p.moveTo(x,y[j]);
	      if(i!=0) p.lineTo(x,y[j]);
	}}
	Path2D.Double q=new Path2D.Double();
	q.moveTo(diag[1].x,diag[1].y);
	q.lineTo(diag[2].x,diag[2].y);
	Path2D.Double[] pp={p,q};
	return pp;
    }

    /**This is for when the verticals to not all hit the conic*/

    public static Path2D.Double[] toPath2(double[] d,double[] vert,Complex[] diag ) {

	Path2D.Double p=new Path2D.Double();
	for(int j=0;j<2;++j) {
	for(int i=0;i<2000;++i) {
	    double x=vert[1]-1.0*i/100-.00001;
	    double[] y=rootsX(d,x);
	    if(y==null) return null;
	      if(i==0) p.moveTo(x,y[j]);
	      if(i!=0) p.lineTo(x,y[j]);
	}}


	for(int j=0;j<2;++j) {
	for(int i=0;i<2000;++i) {
	    double x=vert[2]+1.0*i/100+.00001;
	    double[] y=rootsX(d,x);
	    if(y==null) return null;
	      if(i==0) p.moveTo(x,y[j]);
	      if(i!=0) p.lineTo(x,y[j]);
	}}

	Path2D.Double q=new Path2D.Double();
	q.moveTo(diag[1].x,diag[1].y);
	q.lineTo(diag[2].x,diag[2].y);
	Path2D.Double[] pp={p,q};
	return pp;
    }

    /**We write the conic as
       C y^2 + (B x + E) y + (A x^2 + D x + F)
       and then solve for y in terms of x.
    */


    public static double[] rootsX(double[] d,double x) {
	double A=d[2];
	double B=d[1]*x+d[4];
	double C=d[0]*x*x+d[3]*x+d[5];
	double D=B*B-4*A*C;
	if(D<0) return null;
	double r1=(-B + Math.sqrt(B*B-4*A*C))/(2*A);
	double r2=(-B - Math.sqrt(B*B-4*A*C))/(2*A);
	double[] r={r1,r2};
	return r;
    }


    /**I adapted this code from a program written by Tom Davis.  I found it on the web.
       Given 5 vectors p0,...,p4, representing points in the plane, the routine 
       returns the valuers A,B,C,D,E,F such that  Ax^2 + B xy + C y^2 + D x + E y + F =0.*/

    public static double[] toConic(Vector p0,Vector p1,Vector p2,Vector p3,Vector p4) {

	Vector L0=Vector.cross(p0,p1);
	Vector L1=Vector.cross(p1,p2);
	Vector L2=Vector.cross(p2,p3);
	Vector L3=Vector.cross(p3,p4);
	Vector Q=Vector.cross(L0,L3);

	double A=Q.x[0];
	double B=Q.x[1];
	double C=Q.x[2];

        double a1, a2, b1, b2, c1, c2;
        double x0, x4, y0, y4, w0, w4;
        double aa, bb, cc, dd, ee, ff;
        double y4w0, w4y0, w4w0, y4y0, x4w0, w4x0, x4x0, y4x0, x4y0;
        double a1a2, a1b2, a1c2, b1a2, b1b2, b1c2, c1a2, c1b2, c1c2;


        a1 = L1.x[0]; b1 = L1.x[1]; c1 = L1.x[2];
        a2 = L2.x[0]; b2 = L2.x[1]; c2 = L2.x[2];
        x0 = p0.x[0]; y0 = p0.x[1]; w0 = p0.x[2];
        x4 = p4.x[0]; y4 = p4.x[1]; w4 = p4.x[2];
    
    y4w0 = y4*w0;
    w4y0 = w4*y0;
    w4w0 = w4*w0;
    y4y0 = y4*y0;
    x4w0 = x4*w0;
    w4x0 = w4*x0;
    x4x0 = x4*x0;
    y4x0 = y4*x0;
    x4y0 = x4*y0;
    a1a2 = a1*a2;
    a1b2 = a1*b2;
    a1c2 = a1*c2;
    b1a2 = b1*a2;
    b1b2 = b1*b2;
    b1c2 = b1*c2;
    c1a2 = c1*a2;
    c1b2 = c1*b2;
    c1c2 = c1*c2;

    aa = -A*a1a2*y4w0
	 +A*a1a2*w4y0 
	 -B*b1a2*y4w0
	 -B*c1a2*w4w0
	 +B*a1b2*w4y0
	 +B*a1c2*w4w0
	 +C*b1a2*y4y0
	 +C*c1a2*w4y0
	 -C*a1b2*y4y0
	 -C*a1c2*y4w0;

    cc =  A*c1b2*w4w0
         +A*a1b2*x4w0
	 -A*b1c2*w4w0
	 -A*b1a2*w4x0
	 +B*b1b2*x4w0
	 -B*b1b2*w4x0
	 +C*b1c2*x4w0
	 +C*b1a2*x4x0
	 -C*c1b2*w4x0
	 -C*a1b2*x4x0;

    ff =  A*c1a2*y4x0
	 +A*c1b2*y4y0
	 -A*a1c2*x4y0
	 -A*b1c2*y4y0
	 -B*c1a2*x4x0
	 -B*c1b2*x4y0
	 +B*a1c2*x4x0
	 +B*b1c2*y4x0
	 -C*c1c2*x4y0
	 +C*c1c2*y4x0;

    bb =  A*c1a2*w4w0
	 +A*a1a2*x4w0
	 -A*a1b2*y4w0
	 -A*a1c2*w4w0
	 -A*a1a2*w4x0
	 +A*b1a2*w4y0
	 +B*b1a2*x4w0
	 -B*b1b2*y4w0
	 -B*c1b2*w4w0
	 -B*a1b2*w4x0
	 +B*b1b2*w4y0
	 +B*b1c2*w4w0
	 -C*b1c2*y4w0
	 -C*b1a2*x4y0
	 -C*b1a2*y4x0
	 -C*c1a2*w4x0
	 +C*c1b2*w4y0
	 +C*a1b2*x4y0
	 +C*a1b2*y4x0
	 +C*a1c2*x4w0;

    dd = -A*c1a2*y4w0
	 +A*a1a2*y4x0
	 +A*a1b2*y4y0
	 +A*a1c2*w4y0
	 -A*a1a2*x4y0
	 -A*b1a2*y4y0
	 +B*b1a2*y4x0
	 +B*c1a2*w4x0
	 +B*c1a2*x4w0
	 +B*c1b2*w4y0
	 -B*a1b2*x4y0
	 -B*a1c2*w4x0
	 -B*a1c2*x4w0
	 -B*b1c2*y4w0
	 +C*b1c2*y4y0
	 +C*c1c2*w4y0
	 -C*c1a2*x4y0
	 -C*c1b2*y4y0
	 -C*c1c2*y4w0
	 +C*a1c2*y4x0;

    ee = -A*c1a2*w4x0
	 -A*c1b2*y4w0
	 -A*c1b2*w4y0
	 -A*a1b2*x4y0
	 +A*a1c2*x4w0
	 +A*b1c2*y4w0
	 +A*b1c2*w4y0
	 +A*b1a2*y4x0
	 -B*b1a2*x4x0
	 -B*b1b2*x4y0
	 +B*c1b2*x4w0
	 +B*a1b2*x4x0
	 +B*b1b2*y4x0
	 -B*b1c2*w4x0
	 -C*b1c2*x4y0
	 +C*c1c2*x4w0
	 +C*c1a2*x4x0
	 +C*c1b2*y4x0
	 -C*c1c2*w4x0
	 -C*a1c2*x4x0;

    double[] X={aa,bb,cc,dd,ee,ff};

    double max=0;
    for(int i=0;i<6;++i) {
	if(max<Math.abs(X[i])) max=Math.abs(X[i]);
    }
    for(int i=0;i<6;++i) X[i]=X[i]/max;

    return(X);
    }




}

